import datetime
import time
import logging
import os
import sys

logging.basicConfig(
    format="%(asctime)s - %(levelname)s - [%(filename)s:%(lineno)d] - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
    level=logging.INFO,
)

RETRIES = 30
DELAY_SECONDS = 30
MAX_AGE_MINUTES = int(RETRIES * DELAY_SECONDS / 60) + 5

EXIT_NOT_TRIGGERED = 101
EXIT_RUN_ID_NOT_FOUND = 102
EXIT_RUN_FAIL = 103


class Workflow:
    def __init__(self, github, repo_name, workflow_file, branch):
        self.server_url = os.environ.get("GITHUB_SERVER", "https://github.gwd.broadcom.net")
        self.repo_name = repo_name
        self.workflow_file = workflow_file
        self.branch = branch
        self.repo = github.get_repo(self.repo_name)
        self.workflow = self.repo.get_workflow(self.workflow_file)

    def trigger_workflow_run(self, custom_id, inputs):
        logging.info(
            f"Triggering workflow {self.workflow} with CUSTOM_ID='{custom_id}' for '{inputs}'"
        )

        return self.workflow.create_dispatch(ref=self.branch, inputs=inputs)

    def get_workflow_run(self, custom_id):
        logging.info(f"Searching for workflow run with CUSTOM_ID='{custom_id}'")

        now = datetime.datetime.now()
        no_sooner_than = now - datetime.timedelta(minutes=MAX_AGE_MINUTES)
        runs = self.workflow.get_runs(
            branch=self.branch, created=f">{no_sooner_than.isoformat().split('.')[0]}"
        )

        for run in runs:
            if any(step.name == f"id={custom_id}" for step in run.jobs()[0].steps):
                logging.info(f"Found workflow run with id: {run.id}")

                return run

        logging.info("Workflow run not found (yet)")

        return None

    def start_process(self, custom_id, inputs):
        if not self.trigger_workflow_run(custom_id, inputs):
            logging.error(
                f"Failed to trigger workflow {self.workflow} with CUSTOM_ID='{custom_id}'"
            )

            sys.exit(EXIT_NOT_TRIGGERED)

        for _ in range(RETRIES):
            run = self.get_workflow_run(custom_id)

            if run:
                break

            logging.info(f"Waiting for {DELAY_SECONDS} seconds before trying again")
            time.sleep(DELAY_SECONDS)

        if run is None:
            logging.error("Workflow run not found!")

            sys.exit(EXIT_RUN_ID_NOT_FOUND)

        logging.info(f"Polling for completion every {DELAY_SECONDS} seconds")

        while run.status != "completed":
            time.sleep(DELAY_SECONDS)
            run.update()

        print("\n\n\n")
        print(f"Workflow run {self.workflow} completed with status: {run.conclusion}")
        print(
            f"Workflow run {self.workflow} run URL: {self.server_url}/{self.repo_name}/actions/runs/{run.id}"
        )
        print("\n\n\n")

        if run.conclusion != "success":
            sys.exit(EXIT_RUN_FAIL)
